#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

install_main() {
    clear && tput cnorm

    # unpack the file system
    oneshot install_base

    # generate /etc/fstab and touch it up if we used a swapfile
    genfstab -U $MNT > $MNT/etc/fstab 2>$ERR
    check_for_errors "genfstab -U $MNT > $MNT/etc/fstab"
    [[ -f $MNT/swapfile ]] && sed -i "s~${MNT}~~" $MNT/etc/fstab

    # update the mirrorlist..  MUST be done before updating or it may be slow
    $MIRROR_CMD --verbose --save $MNT/etc/pacman.d/mirrorlist ||
        reflector --score 100 -l 50 -f 10 --sort rate --verbose --save $MNT/etc/pacman.d/mirrorlist

    # MUST be before bootloader and mkinitcpio
    oneshot package_operations

    # set up user login
    oneshot login_manager

    # MUST be done after installing the packages
    run_mkinitcpio
    install_bootloader

    chroot_cmd "hwclock --systohc --utc" || chroot_cmd "hwclock --systohc --utc --directisa"

    oneshot create_user # create the user last to avoid referencing multiple $HOME locations
    return 0
}

install_base() {
    echo -e "\nUnpacking base file system --- Total: ~ 2.8G\n\n"
    rsync -ah --info=progress2 /run/archiso/sfs/airootfs/ $MNT/

    # remove archiso init files and clean up install files
    rm -rf $MNT/etc/sudoers.d/g_wheel
    rm -f $MNT/etc/mkinitcpio-archiso.conf
    rm -f $MNT/etc/polkit-1/rules.d/49-nopasswd_global.rules
    find $MNT/usr/lib/initcpio -name 'archiso*' -type f -exec rm '{}' \;

    # cleanup system permissions
    sed -i 's/volatile/auto/g' $MNT/etc/systemd/journald.conf
    sed -i "s/# %wheel ALL=(ALL) ALL/%wheel ALL=(ALL) ALL/g" $MNT/etc/sudoers

    if [[ $VM ]]; then
        # in a VM remove xorg configs, these cause issues
        rm -rf $MNT/etc/X11/xorg.conf.d
    elif [[ $(lspci | grep ' VGA ' | grep 'Intel') != "" ]]; then
        # xorg config for intel, this should never happen in a VM
        cat > $MNT/etc/X11/xorg.conf.d/20-intel.conf <<EOF
Section "Device"
    Identifier  "Intel Graphics"
    Driver      "intel"
    Option      "TearFree" "true"
EndSection
EOF
    fi

    # copy the kernel image for the regular kernel
    [[ $KERNEL != 'linux-lts' ]] && cp -f $RUN/x86_64/vmlinuz $MNT/boot/vmlinuz-linux

    # copy CPU micro-code if set.  manufacturer_ucode.img -> manufacturer-ucode.img
    [[ $UCODE ]] && cp -f $RUN/${UCODE/-/_} $MNT/boot/$UCODE

    # copy network settings
    cp -rf /etc/NetworkManager/system-connections $MNT/etc/NetworkManager/
    cp -f /etc/resolv.conf $MNT/etc/

    # set the locale and timezone
    sed -i "s/#en_US.UTF-8/en_US.UTF-8/g; s/#${LOCALE}/${LOCALE}/g" $MNT/etc/locale.gen
    sed -i "s/en_US.UTF-8/${LOCALE}/g" $MNT/etc/locale.conf
    cp -f $MNT/etc/locale.conf $MNT/etc/default/locale
    chroot_cmd "locale-gen" 2>/dev/null
    chroot_cmd "ln -sf /usr/share/zoneinfo/$ZONE/$SUBZONE /etc/localtime" 2>/dev/null

    # set the keymaps
    cat > $MNT/etc/X11/xorg.conf.d/00-keyboard.conf <<EOF
# Use localectl(1) to instruct systemd-localed to update it.
Section "InputClass"
    Identifier      "system-keyboard"
    MatchIsKeyboard "on"
    Option          "XkbLayout" "$KEYMAP"
EndSection
EOF
    cat > $MNT/etc/default/keyboard <<EOF
# KEYBOARD CONFIGURATION FILE
# Consult the keyboard(5) manual page.
XKBMODEL=""
XKBLAYOUT="$KEYMAP"
XKBVARIANT=""
XKBOPTIONS=""
BACKSPACE="guess"
EOF
    # console keymap
    cat > $MNT/etc/vconsole.conf <<EOF
KEYMAP=$CONSOLE_MAP
FONT=$FONT
EOF
    # set the hostname
    echo "$HOSTNAME" > $MNT/etc/hostname
    cat > $MNT/etc/hosts << EOF
127.0.0.1   localhost
127.0.1.1   $HOSTNAME
::1     localhost ip6-localhost ip6-loopback
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
EOF
    return 0
}

create_user() {
    # set root password
    chroot_cmd "chpasswd <<< 'root:$(openssl enc -pbkdf2 -a -d -salt -pass pass:$SALT <<< "$ROOT_PASS")'"

    # edit the group and passwd files in /etc/ to swap the liveuser account
    sed -i "s/${LIVE}/${NEWUSER}/g" $MNT/etc/{group,gshadow,passwd,shadow}

    if [[ -d $MNT/home/$NEWUSER ]]; then
        rm -rf $MNT/home/$LIVE
    else
        # fix up some user files that reference the liveuser
        sed -i "s/${LIVE}/${NEWUSER}/g" $MNT/home/$LIVE/.config/gtk-3.0/bookmarks \
            $MNT/home/$LIVE/.mozilla/firefox/archlabs.default/{prefs,sessionstore}.js
        [[ $INSTALL_WMS =~ bspwm ]] || rm -rf $MNT/home/$LIVE/.config/{bspwm,sxhkd}
        [[ $INSTALL_WMS =~ i3-gaps ]] || rm -rf $MNT/home/$LIVE/.config/i3
        [[ $INSTALL_WMS =~ openbox ]] || rm -rf $MNT/home/$LIVE/.config/{openbox,skippy-xd,jgmenu,conky,tint2}
        chroot_cmd "mv -f /home/$LIVE /home/$NEWUSER"
    fi
    chroot_cmd "usermod -aG rfkill,wheel,network,storage,power,video,audio,lp,autologin $NEWUSER"
    chroot_cmd "chpasswd <<< '$NEWUSER:$(openssl enc -pbkdf2 -a -d -salt -pass pass:$SALT <<< "$USER_PASS")'"
    chroot_cmd "chown -Rf $NEWUSER:users /home/$NEWUSER"
}

run_mkinitcpio() {
    local add=""

    # setup a keyfile for LUKS.. Only when choosing grub and system is UEFI
    [[ $LUKS -eq 1 && $LVM -eq 0 && $SYS == 'UEFI' && $BOOTLDR == 'grub' ]] && luks_keyfile

    # new hooks needed in /etc/mkinitcpio.conf if we used LUKS and/or LVM
    (( LVM == 1 )) && add="lvm2"
    (( LUKS == 1 )) && add="encrypt$([[ $add ]] && echo -n " $add")"
    sed -i "s/block filesystems/block ${add} filesystems ${MKINIT_HOOKS}/g" $MNT/etc/mkinitcpio.conf

    chroot_cmd "mkinitcpio -p $KERNEL" 2>$ERR
    check_for_errors "mkinitcpio -p $KERNEL"
}

package_operations() {
    local inpkg="$EXTRA_PACKAGES"  # add the packages chosen during setup
    local rmpkg="archlabs-installer"  # always remove the installer
    local vbx="virtualbox-guest-modules-arch" # save retyping this multiple times

    # the LTS kernel uses different packages
    if [[ $KERNEL == 'linux-lts' ]]; then
        rmpkg="$rmpkg $vbx linux"
        inpkg="$inpkg linux-lts"
        # if the system is a VM then install the needed packages otherwise remove the guest utils
        [[ $VM ]] && inpkg="$inpkg virtualbox-guest-dkms linux-lts-headers" || rmpkg="$rmpkg virtualbox-guest-utils"
    elif ! [[ $VM ]]; then
        # if the system is not a VM then remove the guest utils
        rmpkg="$rmpkg $vbx virtualbox-guest-utils"
    fi

    # for only gnome or cinnamon we don't need the xfce provided stuff
    [[ $INSTALL_WMS == 'gnome' || $INSTALL_WMS == 'cinnamon' ]] && rmpkg="$(pacman -Qssq 'xfce4*' 2>/dev/null)"

    # when not using grub bootloader remove it's package and configurations
    if [[ $BOOTLDR != 'grub' ]]; then
        rmpkg="$rmpkg grub"
        rm -f $MNT/etc/default/grub
        find $MNT/boot/ -name 'grub*' -exec rm -rf '{}' \; >/dev/null 2>&1
    elif [[ $BOOTLDR != 'syslinux' ]]; then
        # do the same when not using syslinux as the bootloader
        find $MNT/boot/ -name 'syslinux*' -exec rm -rf '{}' \; >/dev/null 2>&1
    fi

    # iputils, base-devel, and git are all needed and should always be installed separately
    local base="pacman -S iputils git --noconfirm; pacman -S base-devel --needed --noconfirm"
    local install="pacman -S $inpkg --needed --noconfirm"
    local remove="pacman -Rs $rmpkg --noconfirm"

    chroot_cmd "pacman -Syyu --noconfirm; $base; $install; $remove" 2>/dev/null

    # for neovim copy the default vimrc and colorscheme to ~/.config/nvim
    if [[ $EXTRA_PACKAGES =~ neovim ]]; then
        mkdir -p $MNT/home/$LIVE/.config/nvim
        cp -f $MNT/home/$LIVE/.vimrc $MNT/home/$LIVE/.config/nvim/init.vim
        cp -rf $MNT/home/$LIVE/.vim/colors $MNT/home/$LIVE/.config/nvim/colors
    fi

    # install and setup dwm
    if [[ $INSTALL_WMS =~ dwm ]]; then
        mkdir -pv $MNT/home/$LIVE/suckless
        for prog in dwm dmenu st; do
            chroot_cmd "git clone https://bitbucket.org/natemaia/$prog /home/$LIVE/suckless/$prog && cd /home/$LIVE/suckless/$prog"
            [[ -e $MNT/home/$LIVE/suckless/$prog/config.h ]] && rm -f $MNT/home/$LIVE/suckless/$prog/config.h
            chroot_cmd "make clean install && make clean"
        done
    fi
}

login_manager() {
    if [[ $LOGIN_TYPE == 'lightdm' ]]; then
        rm -rf $MNT/etc/systemd/system/getty@tty1.service.d
        for file in $MNT/home/$LIVE/.{xinitrc,zprofile}; do
            awk '{if($0 && $0 !~ "^#") print "# "$0; else print $0}' $file > $file
        done
        chroot_cmd 'systemctl enable lightdm.service && systemctl set-default graphical.target'
        cat > $MNT/etc/lightdm/lightdm-gtk-greeter.conf << EOF
# LightDM GTK+ Configuration

[greeter]
active-monitor=0
default-user-image=/usr/share/icons/ArchLabs-Dark/64x64/places/distributor-logo-archlabs.png
background=/usr/share/backgrounds/archlabs/archlabs.jpg
theme-name=ArchLabs-dARK
icon-theme-name=Adwaita
font-name=DejaVu Sans Mono 11
position=30%,end 50%,end
EOF
        if [[ $AUTOLOGIN == true ]]; then
            chroot_cmd "groupadd -r nopasswdlogin && usermod -aG nopasswdlogin $NEWUSER"
            sed -i '/#%PAM-1.0/ a auth        sufficient  pam_succeed_if.so user ingroup nopasswdlogin' $MNT/etc/pam.d/lightdm
            sed -i "/#autologin-session=/ c autologin-session=${LOGIN_WM}" $MNT/etc/lightdm/lightdm.conf
            sed -i "/#autologin-user=/ c autologin-user=${NEWUSER}" $MNT/etc/lightdm/lightdm.conf
        fi
    else  # xinit login
        sed -i "s/openbox-session/${LOGIN_WM}/g" $MNT/home/$LIVE/.xinitrc
        sed -i 's|exec sudo archlabs-installer|exec startx -- vt1 &>/dev/null|' $MNT/home/$LIVE/.zprofile
        if [[ $AUTOLOGIN == true ]]; then
            sed -i "s/${LIVE}/${NEWUSER}/g" $MNT/etc/systemd/system/getty@tty1.service.d/autologin.conf
        else
            rm -rf $MNT/etc/systemd/system/getty@tty1.service.d
        fi
    fi
}
